@@ -29,7 +29,7 @@ let MimeHandler = imports.ui.mimeHandler;
 let Gtk = imports.gi.Gtk;
 let Sushi = imports.gi.Sushi;
 
-let Gettext = imports.gettext.domain("sushi");
+let Gettext = imports.gettext.domain('sushi');
 let _ = Gettext.gettext;
 
 let Constants = imports.util.constants;
@@ -41,6 +41,9 @@ function FolderRenderer(args) {
 
 FolderRenderer.prototype = {
     _init : function() {
+        this._folderLoader = null;
+        this._folderLoaderId = 0;
+
         this.moveOnClick = true;
         this.canFullScreen = false;
     },
@@ -54,28 +57,22 @@ FolderRenderer.prototype = {
         this.lastHeight = 0;
 
         this._folderLoader = new Sushi.FileLoader();
-        this._folderLoader.connect("notify::size",
-                                   Lang.bind(this, this._onFolderInfoChanged));
-        this._folderLoader.connect("notify::icon",
-                                   Lang.bind(this, this._onFolderInfoChanged));
-        this._folderLoader.connect("notify::time",
-                                   Lang.bind(this, this._onFolderInfoChanged));
-        this._folderLoader.connect("notify::name",
-                                   Lang.bind(this, this._onFolderInfoChanged));
-
         this._folderLoader.file = file;
+        this._folderLoaderId =
+            this._folderLoader.connect('notify',
+                                       Lang.bind(this, this._onFolderInfoChanged));
 
         this._box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
                                   spacing: 6 });
-        this._image = new Gtk.Image({ "icon-name": "folder",
-                                      "pixel-size": 256 });
+        this._image = new Gtk.Image({ icon_name: 'folder',
+                                      pixel_size: 256 });
         this._box.pack_start(this._image, false, false, 0);
 
         let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
                                  spacing: 1,
-                                 "margin-top": 48,
-                                 "margin-left": 12,
-                                 "margin-right": 12 });
+                                 margin_top: 48,
+                                 margin_left: 12,
+                                 margin_right: 12 });
         this._box.pack_start(vbox, false, false, 0);
 
         let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
@@ -114,23 +111,23 @@ FolderRenderer.prototype = {
 	let name = this._folderLoader.name;
 	if (!name) {
 	    try {
-		name = this._folderLoader.file.get_basename()
+		name = this._folderLoader.file.get_basename();
 	    } catch (e) {
-		name = "";
+		name = '';
 	    }
 	}
         let titleStr =
-            "<b><big>" + name + "</big></b>";
+            '<b><big>' + name + '</big></b>';
 
         let sizeStr =
-            "<small><b>" + _("Size") + "  </b>" +
+            '<small><b>' + _("Size") + '  </b>' +
             ((this._folderLoader.size) ? (this._folderLoader.size) : (_("Loading...")))
-             + "</small>";
+             + '</small>';
 
         let dateStr =
-            "<small><b>" + _("Modified") + "  </b>" +
+            '<small><b>' + _("Modified") + '  </b>' +
              ((this._folderLoader.time) ? (this._folderLoader.time) : (_("Loading...")))
-             + "</small>";
+             + '</small>';
 
         this._titleLabel.set_markup(titleStr);
         this._sizeLabel.set_markup(sizeStr);
@@ -149,8 +146,13 @@ FolderRenderer.prototype = {
     },
 
     clear : function() {
-        this._folderLoader.stop();
-        delete this._folderLoader;
+        if (this._folderLoader) {
+            this._folderLoader.disconnect(this._folderLoaderId);
+            this._folderLoaderId = 0;
+
+            this._folderLoader.stop();
+            this._folderLoader = null;
+        }
     },
 
     getSizeForAllocation : function(allocation) {
@@ -161,4 +163,4 @@ FolderRenderer.prototype = {
 let handler = new MimeHandler.MimeHandler();
 let renderer = new FolderRenderer();
 
-handler.registerMime("inode/directory", renderer);
+handler.registerMime('inode/directory', renderer);
