@@ -28,7 +28,6 @@
 #include "sushi-font-widget.h"
 #include "sushi-font-loader.h"
 
-#include <cairo/cairo-ft.h>
 #include <math.h>
 
 enum {
@@ -44,6 +43,7 @@ enum {
 struct _SushiFontWidgetPrivate {
   gchar *uri;
 
+  FT_Library library;
   FT_Face face;
   gchar *face_contents;
 
@@ -65,6 +65,8 @@ G_DEFINE_TYPE (SushiFontWidget, sushi_font_widget, GTK_TYPE_DRAWING_AREA);
 #define SURFACE_SIZE 4
 #define SECTION_SPACING 16
 
+#define TITLE_SIZE 6
+
 static const gchar lowercase_text_stock[] = "abcdefghijklmnopqrstuvwxyz";
 static const gchar uppercase_text_stock[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 static const gchar punctuation_text_stock[] = "0123456789.:,;(*!?')";
@@ -82,12 +84,15 @@ draw_string (cairo_t *cr,
 	     const gchar *text,
 	     gint *pos_y)
 {
+  cairo_font_extents_t font_extents;
   cairo_text_extents_t extents;
 
+  cairo_font_extents (cr, &font_extents);
   cairo_text_extents (cr, text, &extents);
 
   if (pos_y != NULL)
-    *pos_y += extents.height + extents.y_advance + padding.top;
+    *pos_y += font_extents.ascent + font_extents.descent +
+      extents.y_advance + padding.top;
 
   cairo_move_to (cr, padding.left, *pos_y);
   cairo_show_text (cr, text);
@@ -226,7 +231,7 @@ build_sizes_table (FT_Face face,
 
   /* work out what sizes to render */
   if (FT_IS_SCALABLE (face)) {
-    *n_sizes = 8;
+    *n_sizes = 14;
     sizes = g_new (gint, *n_sizes);
     sizes[0] = 8;
     sizes[1] = 10;
@@ -236,6 +241,12 @@ build_sizes_table (FT_Face face,
     sizes[5] = 36;
     sizes[6] = 48;
     sizes[7] = 72;
+    sizes[8] = 96;
+    sizes[9] = 120;
+    sizes[10] = 144;
+    sizes[11] = 168;
+    sizes[12] = 192;
+    sizes[13] = 216;
     *alpha_size = 24;
   } else {
     /* use fixed sizes */
@@ -258,12 +269,14 @@ build_sizes_table (FT_Face face,
 static void
 sushi_font_widget_size_request (GtkWidget *drawing_area,
                                 gint *width,
-                                gint *height)
+                                gint *height,
+                                gint *min_height)
 {
   SushiFontWidget *self = SUSHI_FONT_WIDGET (drawing_area);
   SushiFontWidgetPrivate *priv = self->priv;
   gint i, pixmap_width, pixmap_height;
   cairo_text_extents_t extents;
+  cairo_font_extents_t font_extents;
   cairo_font_face_t *font;
   gint *sizes = NULL, n_sizes, alpha_size;
   cairo_t *cr;
@@ -278,6 +291,8 @@ sushi_font_widget_size_request (GtkWidget *drawing_area,
       *width = 1;
     if (height != NULL)
       *height = 1;
+    if (min_height != NULL)
+      *min_height = 1;
 
     return;
   }
@@ -296,39 +311,40 @@ sushi_font_widget_size_request (GtkWidget *drawing_area,
   pixmap_height = 0;
 
   font = cairo_ft_font_face_create_for_ft_face (face, 0);
-
-  if (self->priv->font_supports_title)
-    cairo_set_font_face (cr, font);
-
-  cairo_set_font_size (cr, alpha_size + 6);
-  cairo_text_extents (cr, self->priv->font_name, &extents);
-  pixmap_height += extents.height + extents.y_advance + padding.top + padding.bottom;
-  pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
-
-  if (!self->priv->font_supports_title)
-    cairo_set_font_face (cr, font);
-
+  cairo_set_font_face (cr, font);
   cairo_font_face_destroy (font);
 
-  pixmap_height += SECTION_SPACING / 2;
+  if (self->priv->font_supports_title) {
+      cairo_set_font_size (cr, sizes[TITLE_SIZE]);
+      cairo_font_extents (cr, &font_extents);
+      cairo_text_extents (cr, self->priv->font_name, &extents);
+      pixmap_height += font_extents.ascent + font_extents.descent +
+        extents.y_advance + padding.top + padding.bottom;
+      pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
+  }
 
+  pixmap_height += SECTION_SPACING / 2;
   cairo_set_font_size (cr, alpha_size);
+  cairo_font_extents (cr, &font_extents);
 
   if (self->priv->lowercase_text != NULL) {
     cairo_text_extents (cr, self->priv->lowercase_text, &extents);
-    pixmap_height += extents.height + extents.y_advance + padding.top + padding.bottom;
+    pixmap_height += font_extents.ascent + font_extents.descent + 
+      extents.y_advance + padding.top + padding.bottom;
     pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
   }
 
   if (self->priv->uppercase_text != NULL) {
     cairo_text_extents (cr, self->priv->uppercase_text, &extents);
-    pixmap_height += extents.height + extents.y_advance + padding.top + padding.bottom;
+    pixmap_height += font_extents.ascent + font_extents.descent +
+      extents.y_advance + padding.top + padding.bottom;
     pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
   }
 
   if (self->priv->punctuation_text != NULL) {
     cairo_text_extents (cr, self->priv->punctuation_text, &extents);
-    pixmap_height += extents.height + extents.y_advance + padding.top + padding.bottom;
+    pixmap_height += font_extents.ascent + font_extents.descent +
+      extents.y_advance + padding.top + padding.bottom;
     pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
   }
 
@@ -336,9 +352,14 @@ sushi_font_widget_size_request (GtkWidget *drawing_area,
 
   for (i = 0; i < n_sizes; i++) {
     cairo_set_font_size (cr, sizes[i]);
+    cairo_font_extents (cr, &font_extents);
     cairo_text_extents (cr, self->priv->sample_string, &extents);
-    pixmap_height += extents.height + extents.y_advance + padding.top + padding.bottom;
+    pixmap_height += font_extents.ascent + font_extents.descent +
+      extents.y_advance + padding.top + padding.bottom;
     pixmap_width = MAX (pixmap_width, extents.width + padding.left + padding.right);
+
+    if ((i == 7) && (min_height != NULL))
+      *min_height = pixmap_height;
   }
 
   pixmap_height += padding.bottom + SECTION_SPACING;
@@ -361,7 +382,7 @@ sushi_font_widget_get_preferred_width (GtkWidget *drawing_area,
 {
   gint width;
 
-  sushi_font_widget_size_request (drawing_area, &width, NULL);
+  sushi_font_widget_size_request (drawing_area, &width, NULL, NULL);
 
   *minimum_width = *natural_width = width;
 }
@@ -371,11 +392,12 @@ sushi_font_widget_get_preferred_height (GtkWidget *drawing_area,
                                         gint *minimum_height,
                                         gint *natural_height)
 {
-  gint height;
+  gint height, min_height;
 
-  sushi_font_widget_size_request (drawing_area, NULL, &height);
+  sushi_font_widget_size_request (drawing_area, NULL, &height, &min_height);
 
-  *minimum_height = *natural_height = height;
+  *minimum_height = min_height;
+  *natural_height = height;
 }
 
 static gboolean
@@ -391,6 +413,7 @@ sushi_font_widget_draw (GtkWidget *drawing_area,
   GdkRGBA color;
   GtkBorder padding;
   GtkStateFlags state;
+  gint allocated_height;
 
   if (face == NULL)
     goto end;
@@ -405,37 +428,46 @@ sushi_font_widget_draw (GtkWidget *drawing_area,
   sizes = build_sizes_table (face, &n_sizes, &alpha_size);
 
   font = cairo_ft_font_face_create_for_ft_face (face, 0);
-
-  if (self->priv->font_supports_title)
-    cairo_set_font_face (cr, font);
-
-  /* draw text */
-  cairo_set_font_size (cr, alpha_size + 6);
-  draw_string (cr, padding, self->priv->font_name, &pos_y);
-
-  if (!self->priv->font_supports_title)
-    cairo_set_font_face (cr, font);
-
+  cairo_set_font_face (cr, font);
   cairo_font_face_destroy (font);
 
-  pos_y += SECTION_SPACING / 2;
+  allocated_height = gtk_widget_get_allocated_height (drawing_area);
 
+  /* draw text */
+
+  if (self->priv->font_supports_title) {
+    cairo_set_font_size (cr, sizes[TITLE_SIZE]);
+    draw_string (cr, padding, self->priv->font_name, &pos_y);
+  }
+
+  if (pos_y > allocated_height)
+    goto end;
+
+  pos_y += SECTION_SPACING / 2;
   cairo_set_font_size (cr, alpha_size);
 
   if (self->priv->lowercase_text != NULL)
     draw_string (cr, padding, self->priv->lowercase_text, &pos_y);
+  if (pos_y > allocated_height)
+    goto end;
 
   if (self->priv->uppercase_text != NULL)
     draw_string (cr, padding, self->priv->uppercase_text, &pos_y);
+  if (pos_y > allocated_height)
+    goto end;
 
   if (self->priv->punctuation_text != NULL)
     draw_string (cr, padding, self->priv->punctuation_text, &pos_y);
+  if (pos_y > allocated_height)
+    goto end;
 
   pos_y += SECTION_SPACING;
 
   for (i = 0; i < n_sizes; i++) {
     cairo_set_font_size (cr, sizes[i]);
     draw_string (cr, padding, self->priv->sample_string, &pos_y);
+    if (pos_y > allocated_height)
+      break;
   }
 
  end:
@@ -474,7 +506,8 @@ font_face_async_ready_cb (GObject *object,
 static void
 load_font_face (SushiFontWidget *self)
 {
-  sushi_new_ft_face_from_uri_async (self->priv->uri,
+  sushi_new_ft_face_from_uri_async (self->priv->library,
+                                    self->priv->uri,
                                     font_face_async_ready_cb,
                                     self);
 }
@@ -492,10 +525,16 @@ sushi_font_widget_set_uri (SushiFontWidget *self,
 static void
 sushi_font_widget_init (SushiFontWidget *self)
 {
+  FT_Error err;
+
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, SUSHI_TYPE_FONT_WIDGET,
                                             SushiFontWidgetPrivate);
 
   self->priv->face = NULL;
+  err = FT_Init_FreeType (&self->priv->library);
+
+  if (err != FT_Err_Ok)
+    g_error ("Unable to initialize FreeType");
 }
 
 static void
@@ -550,6 +589,11 @@ sushi_font_widget_finalize (GObject *object)
   g_free (self->priv->sample_string);
   g_free (self->priv->face_contents);
 
+  if (self->priv->library != NULL) {
+    FT_Done_FreeType (self->priv->library);
+    self->priv->library = NULL;
+  }
+
   G_OBJECT_CLASS (sushi_font_widget_parent_class)->finalize (object);
 }
 
@@ -591,3 +635,13 @@ sushi_font_widget_new (const gchar *uri)
                        "uri", uri,
                        NULL);
 }
+
+/**
+ * sushi_font_widget_get_ft_face: (skip)
+ *
+ */
+FT_Face
+sushi_font_widget_get_ft_face (SushiFontWidget *self)
+{
+  return self->priv->face;
+}
